const {makeExecutableSchema} = require("@graphql-tools/schema");

const UserModel = require("../models/User");

const axios = require('axios');

const {
    addAdWithPhotos,
    updateAd,
    getAdById,
    getAds,
    deleteAd,
    askQuestion,
    answerQuestion,
} = require("../service/AdService");

const typeDefs = `
  type Ad {
    _id: ID
    title: String!
    propertyType: String!
    publicationStatus: String!
    propertyStatus: String!
    description: String!
    price: Float!
    availabilityDate: String!
    photos: [String]
    userName: String!
    questions: [Question]
  }

  type Question {
    user: String!
    question: String!
    answers: [Answer]
    date: String!
  }

  type Answer {
    answer: String!
  }

  type AuthPayload {
    token: String!
  }

  type Query {
    getAds(token: String): [Ad]
    getAdById(adId: ID!, token: String): Ad
    getUser(id: ID, username: String): User
  }

  type Mutation {
    addAd(input: AdInput, photos: [String], token: String): Ad
    updateAd(adId: ID!, input: AdInput, photos: [String], token: String): Ad
    deleteAd(adId: ID!, token: String): String
    askQuestion(adId: ID!, question: String!, token: String): Ad
    answerQuestion(adId: ID!, questionId: ID!, answer: String!, token: String): Ad
    createUser(input: UserInput): User
    loginUser(username: String!, password: String!): AuthPayload
    logoutUser: String
  }

  input AdInput {
    title: String!
    propertyType: String!
    publicationStatus: String!
    propertyStatus: String!
    description: String!
    price: Float!
    availabilityDate: String!
  }
  
  type User {
    _id: ID!
    username: String!
    password: String
    isAgent: Boolean
  }

  input UserInput {
    username: String!
    password: String
    isAgent: Boolean
  }
`;

const resolvers = {
    Query: {
        getAds: async (_, context) => {
            const tokenHeader = context.headers.authorization;

            if (!tokenHeader || !tokenHeader.startsWith('Bearer ')) {
                throw new Error('L\'en-tête d\'autorisation est manquant ou a un format invalide');
            }
            const token = context.headers.authorization.replace('Bearer ', '');
            const result = await getAds(token);
            return result;
        },
        getAdById: async (_, {adId}, context) => {
            const tokenHeader = context.headers.authorization;

            if (!tokenHeader || !tokenHeader.startsWith('Bearer ')) {
                throw new Error('L\'en-tête d\'autorisation est manquant ou a un format invalide');
            }

            const token = context.headers.authorization.replace('Bearer ', '');
            const result = await getAdById(adId, token);

            if (result.status) {
                throw new Error(result.message);
            }

            return result;
        },
        getUser: async (parent, {id, username}) => {
            if (id) {
                return await UserModel.findById(id);
            } else if (username) {
                return await UserModel.findOne({username});
            } else {
                return null;
            }
        },
    },
    Mutation: {
        addAd: async (_, {input, photos}, context) => {
            const tokenHeader = context.headers.authorization;

            if (!tokenHeader || !tokenHeader.startsWith('Bearer ')) {
                throw new Error('L\'en-tête d\'autorisation est manquant ou a un format invalide');
            }

            const token = context.headers.authorization.replace('Bearer ', '');
            const result = await addAdWithPhotos(input, photos, token);

            if (result.status) {
                throw new Error(result.message);
            }

            return result;
        },
        updateAd: async (_, {adId, input, photos}, context) => {
            const tokenHeader = context.headers.authorization;

            if (!tokenHeader || !tokenHeader.startsWith('Bearer ')) {
                throw new Error('L\'en-tête d\'autorisation est manquant ou a un format invalide');
            }

            const token = context.headers.authorization.replace('Bearer ', '');
            const result = await updateAd(adId, input, photos, token);

            if (result.status) {
                throw new Error(result.message);
            }

            return result;
        },
        deleteAd: async (_, {adId}, context) => {
            const tokenHeader = context.headers.authorization;

            if (!tokenHeader || !tokenHeader.startsWith('Bearer ')) {
                throw new Error('L\'en-tête d\'autorisation est manquant ou a un format invalide');
            }

            const token = context.headers.authorization.replace('Bearer ', '');
            const result = await deleteAd(adId, token);

            if (result.status) {
                throw new Error(result.message);
            }

            return result;
        },
        askQuestion: async (_, {adId, question}, context) => {
            const tokenHeader = context.headers.authorization;

            if (!tokenHeader || !tokenHeader.startsWith('Bearer ')) {
                throw new Error('L\'en-tête d\'autorisation est manquant ou a un format invalide');
            }

            const token = context.headers.authorization.replace('Bearer ', '');
            const result = await askQuestion(adId, question, token);

            if (result.status) {
                throw new Error(result.message);
            }

            return result;
        },
        answerQuestion: async (_, {adId, questionId, answer}, context) => {
            const tokenHeader = context.headers.authorization;

            if (!tokenHeader || !tokenHeader.startsWith('Bearer ')) {
                throw new Error('L\'en-tête d\'autorisation est manquant ou a un format invalide');
            }

            const token = context.headers.authorization.replace('Bearer ', '');
            const result = await answerQuestion(adId, questionId, answer, token);

            if (result.status) {
                throw new Error(result.message);
            }

            return result;
        },
        createUser: async (_, {input}) => {
            try {
                const response = await axios.post('http://localhost:8080/user', input);
                if (response.data.exports) {
                    throw new Error(response.data.exports.error);
                }
                return response.data.user;
            } catch (error) {
                console.error('Erreur lors de la création de l\'utilisateur:', error);
                throw new Error('Erreur lors de la création de l\'utilisateur. Vérifiez les détails dans les journaux.');
            }
        },
        loginUser: async (_, {username, password}) => {
            try {
                const response = await axios.post('http://localhost:8080/user/login', {username, password});

                if (response.status === 200) {
                    return {token: response.data.token};
                } else {
                    throw new Error('Invalid username or password.');
                }
            } catch (error) {
                console.error('Erreur lors de la tentative de connexion:', error);
                throw new Error('Erreur lors de la tentative de connexion. Vérifiez les détails dans les journaux.');
            }
        },
        logoutUser: async () => {
            try {
                const response = await axios.post('http://localhost:8080/user/logout');

                if (response.status === 200) {
                    return 'Logout successful.';
                } else {
                    throw new Error('Logout failed.');
                }
            } catch (error) {
                console.error('Erreur lors de la déconnexion de l\'utilisateur:', error);
                throw new Error('Erreur lors de la déconnexion de l\'utilisateur. Vérifiez les détails dans les journaux.');
            }
        },
    },
};

exports.schema = makeExecutableSchema({typeDefs, resolvers});
