const fs = require('fs');
const path = require('path');

const Ad = require('../models/Ad');
const User = require('../models/User');

const {extractUserFromToken} = require("./UserService");

const secretKey = process.env.SECRET_KEY;

// Service for adding an ad
exports.addAdWithPhotos = async function (body, photos, token) {
    try {
        const {title, propertyType, publicationStatus, propertyStatus, description, price, availabilityDate} = body;
        const decoded = await extractUserFromToken(token, secretKey);
        const user = await User.findOne({username: decoded.username});

        if (user.isAgent === false) {
            return {status: 403, message: 'User non autorisé'};
        }

        const photoPaths = [];

        photos.forEach(file => {
            photoPaths.push(file.filename);
        });

        const newAd = new Ad({
            title,
            propertyType,
            publicationStatus,
            propertyStatus,
            description,
            price,
            availabilityDate,
            photos: photoPaths,
            userName: user.username
        });

        const ad = await newAd.save();

        return ad;
    } catch (error) {
        throw error;
    }
};

// Service for getting ads
exports.getAds = async function (token) {
    try {
        const decoded = await extractUserFromToken(token, secretKey);
        let ads;
        const user = await User.findOne({username: decoded.username});
        if (user && user.isAgent) {
            ads = await Ad.find({userName: user.username});
        } else {
            ads = await Ad.find({publicationStatus: 'Publiée'});
        }
        return ads;
    } catch (error) {
        throw error;
    }
};

// Service for adding photos to an ad
exports.addPhotosToAd = async function (adId, photos, token) {
    try {
        const decoded = await extractUserFromToken(token, secretKey);
        const user = await User.findOne({username: decoded.username});
        const ad = await Ad.findById(adId);
        if (!ad) {
            return {status: 404, message: 'Ad not found.'};
        }

        if (user.username !== ad.userName) {
            return {status: 403, message: 'User non autorisé'};
        }

        const photoPaths = [];

        photos.forEach(file => {
            photoPaths.push(file.filename);
        });

        const updatedAd = await Ad.findByIdAndUpdate(
            adId,
            {
                $push: {photos: {$each: photoPaths}},
            },
            {new: true}
        );

        return updatedAd;
    } catch (error) {
        throw error;
    }
};

// Service for updating an ad with photos
exports.updateAd = async function (adId, reqBody, files, token) {
    try {
        const decoded = await extractUserFromToken(token, secretKey);
        const user = await User.findOne({username: decoded.username});
        const ad = await Ad.findById(adId);

        if (!ad) {
            return {status: 404, message: 'Ad not found.'};
        }

        if (user.username !== ad.userName) {
            return {status: 403, message: 'User non autorisé'};
        }

        const {title, propertyType, propertyStatus, publicationStatus, description, price, availabilityDate} = reqBody;

        const photoPaths = [];

        files.forEach(file => {
            photoPaths.push(file.filename);
        });

        const updateAd = await Ad.findByIdAndUpdate(
            adId,
            {
                title,
                propertyType,
                publicationStatus,
                propertyStatus,
                description,
                price,
                availabilityDate,
                photos: photoPaths
            },
            {new: true}
        );


        return updateAd;

    } catch (error) {
        throw error;
    }
};

// Service for deleting an ad
exports.deleteAd = async function (adId, token) {
    try {
        const decoded = await extractUserFromToken(token, secretKey);
        const user = await User.findOne({username: decoded.username});

        const ad = await Ad.findById(adId);

        if (!ad) {
            return {status: 404, message: 'Ad not found.'};
        }

        if (user.username !== ad.userName) {
            return {status: 403, message: 'User non autorisé'};
        }

        const deletedAd = await Ad.findByIdAndRemove(adId);

        deletedAd.photos.forEach((photo) => {
            const filePath = path.join('uploads', photo);
            if (fs.existsSync(filePath)) {
                fs.unlinkSync(filePath);
            }
        });

        return JSON.stringify('Annonce supprimée avec succès');
    } catch (error) {
        throw error;
    }
};

// Service for getting an ad by id
exports.getAdById = async function (adId, token) {
    try {
        const decoded = await extractUserFromToken(token, secretKey);
        const user = await User.findOne({username: decoded.username});

        const adDetails = await Ad.findById(adId);

        if (!adDetails) {
            return {status: 404, message: 'Cette annonce est introuvable ou non disponible.'};
        }

        if (
            (user && user.username !== adDetails.userName || !user) &&
            adDetails.publicationStatus !== 'Publiée'
        ) {
            return {status: 404, message: 'Cette annonce est introuvable ou non disponible.'};
        }
        if (user && user.username !== adDetails.userName && user.isAgent) {
            return {status: 403, message: 'User non autorisée.'};
        }

        return adDetails;
    } catch (error) {
        throw error;
    }
};

// Service for asking a question on an ad
exports.askQuestion = async function (adId, question, token) {
    try {
        const decoded = await extractUserFromToken(token, secretKey);
        const user = await User.findOne({username: decoded.username});

        const ad = await Ad.findById(adId);
        if (!ad) {
            return {status: 404, message: 'Annonce non trouvée.'};
        }
        if (ad.propertyStatus !== 'Disponible') {
            return {status: 404, message: 'Vous ne pouvez poser une question que sur les annonces non disponibles.'};
        }
        ad.questions.push({
            user: user.username,
            question
        });
        const newAd = await ad.save();
        return newAd;
    } catch (error) {
        throw error;
    }
};

// Service for answering a question on an ad
exports.answerQuestion = async function (adId, questionId, answer, token) {
    try {
        const decoded = await extractUserFromToken(token, secretKey);
        const user = await User.findOne({username: decoded.username});

        const ad = await Ad.findById(adId);
        if (!ad) {
            return {status: 404, message: 'Annonce non trouvée.'};
        }

        if (user.username !== ad.userName) {
            return {status: 403, message: 'User non autorisée.'};
        }

        const question = ad.questions.id(questionId);
        if (!question) {
            return {status: 404, message: 'Question non trouvée.'};
        }
        question.answers.push({user: user.username, answer});
        const newAd = await ad.save();
        return newAd;
    } catch (error) {
        throw error;
    }
}
